@@ -33,7 +33,7 @@ setAs(from = "denseMatrix", to = "generalMatrix", as_geSimpl)
         else if (extends(cld,"lMatrix")) as(r, "ltCMatrix")
         else if (extends(cld,"nMatrix")) as(r, "ntCMatrix")
         else if (extends(cld,"zMatrix")) as(r, "ztCMatrix")
-        else stop("undefined method for class ", cl)
+	else stop(gettextf("undefined method for class %s", dQuote(cl)), domain=NA)
     }
 }
 
@@ -98,7 +98,7 @@ setMethod("[", signature(x = "denseMatrix", i = "index", j = "missing",
 		  r <- as(x, "matrix")[i, drop=drop]
 	      else if(na == 4)
 		  r <- as(x, "matrix")[i, , drop=drop]
-	      else stop("invalid nargs()= ",na)
+	      else stop(gettextf("invalid nargs()= %d", na), domain=NA)
 	      if(is.null(dim(r))) r else as(r, geClass(x))
 	  })
 
@@ -161,7 +161,7 @@ setReplaceMethod("[", signature(x = "denseMatrix", i = "index", j = "missing",
 			 r[i] <- value
 		     else if(na == 4)
 			 r[i, ] <- value
-		     else stop("invalid nargs()= ",na)
+		     else stop(gettextf("invalid nargs()= %d", na), domain=NA)
 		     as(r, geClass(x))
 		 })
 
@@ -199,7 +199,7 @@ setMethod("isSymmetric", signature(object = "denseMatrix"),
 	      if (is(object,"dMatrix"))
 		  isTRUE(all.equal(as(object, "dgeMatrix"),
 				   as(t(object), "dgeMatrix"),
-				   tol = tol, ...))
+				   tolerance = tol, ...))
 	      else if (is(object, "nMatrix"))
 		  identical(as(object, "ngeMatrix"),
 			    as(t(object), "ngeMatrix"))
@@ -215,18 +215,14 @@ setMethod("isSymmetric", signature(object = "denseMatrix"),
 			    as(t(object), "igeMatrix"))
 	  })
 
-setMethod("isTriangular", signature(object = "triangularMatrix"),
-	  function(object, ...) TRUE)
+## rather methods in ./triangularMatrix.R
+## setMethod("isTriangular", signature(object = "triangularMatrix"),
+## 	  function(object, ...) TRUE)
 
 setMethod("isTriangular", signature(object = "denseMatrix"), isTriMat)
 
 setMethod("isDiagonal", signature(object = "denseMatrix"), .is.diagonal)
 
-## FIXME: Once we have integer (idense..),  sign(), abs(.) may need different:
-setMethod("Math", signature(x = "denseMatrix"),
-	  function(x) callGeneric(as(x, "dMatrix")))
-                                        # -> ./ddenseMatrix.R has next method
-
 setMethod("rcond", signature(x = "denseMatrix", norm = "character"),
 	  function(x, norm, ...)
 	  rcond(as(as(x, "dMatrix"), "dgeMatrix"), norm=norm, ...))
